@@ -130,6 +130,7 @@ class LexerATNSimulator(ATNSimulator):
         self.startIndex = -1
         self.line = 1
         self.column = 0
+        from antlr4.Lexer import Lexer
         self.mode = Lexer.DEFAULT_MODE
 
     def matchATN(self, input:InputStream):
@@ -539,9 +540,6 @@ class LexerATNSimulator(ATNSimulator):
     # configuration containing an ATN rule stop state. Later, when
     # traversing the DFA, we will know which rule to accept.
     def addDFAState(self, configs:ATNConfigSet) -> DFAState:
-        # the lexer evaluates predicates on-the-fly; by this point configs
-        # should not contain any configurations with unevaluated predicates.
-        assert not configs.hasSemanticContext
 
         proposed = DFAState(configs=configs)
         firstConfigWithRuleStopState = None
